<?php

/***
 * Class OcCacheHelper
 *
 * Helper class for accessing the cache holding the information about witch queries a article is originated in.
 * This to be able to know what cache keys to look in when a update or delete event is pushed from OC notifier.
 *
 * The cache follows a structure such as 'uuid' => array( transientkey => expire time );
 */
class OcCacheHelper {

    const OPTION_KEY = 'oc_cache_map';
    const WIDGET_OPTION_KEY = 'oc_widget_cache_map';
    const MAX_EXPIRE = 1209600;

    private static $instance;

    public static function getInstance() {
        if( self::$instance === null ) {
            self::$instance = new OcCacheHelper();
        }

        return self::$instance;
    }

    private function build_key( $prefix, $uuid ) {
        return join('_', [$prefix, $uuid]);
    }

    public function add_to_cache_map( $uuid, $key, $expire_time ) {
        if( empty( $uuid ) || empty( $key ) ) {
            return;
        }

        $cache = get_transient( $this->build_key(self::OPTION_KEY, $uuid) );
        $expire = time() + $expire_time;

        if( !is_array($cache) ) {
            $cache = [];
        }

        $cache[$key] = $expire;
        set_transient( $this->build_key(self::OPTION_KEY, $uuid), $cache, $expire_time );
    }

    public function remove_from_cache_map( $uuid, $key ) {
        $cache = get_transient( $this->build_key(self::OPTION_KEY, $uuid) );

        if( isset( $cache[$key] ) ) {
            unset( $cache[$key] );

            if( empty( $cache ) ) {
                delete_transient( $this->build_key(self::OPTION_KEY, $uuid) );
                return;
            }
        }

        set_transient( $this->build_key(self::OPTION_KEY, $uuid), $cache, self::MAX_EXPIRE);
    }


    public function get_transient_keys_by_uuid( $uuid ) {
        $ret = array();

        if( false !== ($cache = get_transient( $this->build_key(self::OPTION_KEY, $uuid) ) ) ) {
            $ret = $cache;
        }

        return $ret;
    }

    public function update_cache_map( $uuid, $key, $expire_time ) {
        if( empty( $uuid ) || empty( $key ) )
        return;

        $cache = get_transient( $this->build_key(self::OPTION_KEY, $uuid) );
        $expire = time() + $expire_time;

        if( isset( $cache[$key] ) ) {
            $cache[$key] = $expire;
        }

        set_transient( $this->build_key(self::OPTION_KEY, $uuid), $cache, $expire_time );
        $this->clean_up_cache_map($uuid);
    }

    public function clean_up_cache_map( $uuid ) {
        $time_now = time();

        $cache = get_transient( $this->build_key(self::OPTION_KEY, $uuid) );
        if( $cache ) {
            foreach( $cache as $key => $expire ) {
                if( $time_now >= $expire ) {
                    $this->remove_from_cache_map($uuid, $key);
                }
            }
        }
    }

    /*
     *  Widget cache
     */
    public function create_widget_cache_buster( $key ) {
        set_transient( $this->build_key( self::WIDGET_OPTION_KEY, $key ), true, 4 * WEEK_IN_SECONDS);
    }

    public function clear_widget_cache_by_uuid( $uuid ) {
        $queries = $this->get_transient_keys_by_uuid($uuid);

        foreach($queries as $cache_key => $expire) {
            $widget_cache = get_transient( $this->build_key(self::WIDGET_OPTION_KEY, $cache_key) );
            if( $widget_cache ) {
                delete_transient(OcAPI::TRANSIENT_KEY.'_'.$cache_key);
            }
        }
    }

    public function remove_cache_buster_key( $key ) {
        delete_transient( $this->build_key( self::WIDGET_OPTION_KEY, $key ) );
    }
}